;	ACC_202_88C.ASM
;	ADXL 202 ACCELEROMETER G Force Display
;	PIC 16F88 
;	Y axis read only
;	8 MHz Internal clock (TMR1 tweaked for accuracy)
;	Right justified AD (10 bit)
;	2-Dual 14 segment common cathode LED output (ITC CID)
;	
;	7/4/08			Added Test Point
;------------------------------------------------------------
;	Tested Functional
;------------------------------------------------------------
;
	LIST	P=16F88	
	ERRORLEVEL	-302
	include	<P16F88.INC>
	__CONFIG    _CONFIG1, h'3f10'
	__CONFIG    _CONFIG2, h'0000'
;
;------------------------------------------------------------
;               Port Set-up
;           0   1   2   3   4   5   6   7       bit
; port  A   A/D Tp  IO0 IO1 Out In  IO2 IO3	
;       B   LED LED LED LED LED LED LED LED
;------------------------------------------------------------
;
	ORG	0			; start code
	goto	main
	ORG	4
	goto	int_service
;
;--------------------------------------------------------------------------
;	Pinout is for my LED display demo board (pin #1 far right)	
;
;
;PIN |  1	2	3	4	5	6	7	8 
;    |
;SEG | 	A	B	C	D	E	F	G	DP
;    |
;BIT |	7	6	5	4	3	2	1	0	
;    |	
; 0  | 	1	1	1	1	1	1	0	0	0xFC (0xfd)
;(I) | 	0	0	0	0	0	0	1	1	0x03
;    |
; 1  |	0	1	1	0	0	0	0	0	0x60 (0x61)
;(I) |	1	0	0	1	1	1	1	1	0x9F
;    |
; 2  |	1	1	0	1	1	0	1	0	0xDA (0xdb)
;(I) |	0	0	1	0	0	1	0	1	0x25
;    |
; 3  |	1	1	1	1	0	0	1	0	0xF2 (0xf3)
;(I) |	0	0	0	0	1	1	0	1	0x0D
;    |
; 4  |	0	1	1	0	0	1	1	0	0x66 (0x67)
;(I) |	1	0	0	1	1	0	0	1	0x99
;    |
; 5  |	1	0	1	1	0	1	1	0	0xB6 (0xb7)
;(I) |	0	1	0	0	1	0	0	1	0x49
;    |
; 6  |	1	0	1	1	1	1	1	0	0xBE (0xbf)
;(I) | 	0	1	0	0	0	0	0	1	0x41
;    |
; 7  |	1	1	1	0	0	0	0	0	0xE0 (0xe1)
;(I) |	0	0	0	1	1	1	1	1	0x1F
;    |
; 8  |	1	1	1	1	1	1	1	0	0xFE (0xff)
;(I) |	0	0	0	0	0	0	0	1	0x01
;    |
; 9  |	1	1	1	0	0	1	1	0	0xE6 (0xe7)
;(I) |	0	0	0	1	1	0	0	1	0x19
;    |		
; DP |	0	0	0	0	0	0	0	1	0x01 	
;(I) |	1	1	1	1	1	1	1	0	0xFE
;    |
;(-) |	0	0	0	0	0	0	1	0	0x02 
;(I) | 	1	1	1	1	1	1	0	1	0xFD
;
;	LED Blanking
;SEG	A	B	C	D	E	F	G	.
;	7f	bf	df	ef	f7	fb	fd	fe	
;(I)	80	40	20	10	08	04	02	01		
;------------------------------------------------------------
;	0-9 on a 14 segment LED w/decimal point on
LED_digits
        addwf   PCL,F
        dt 	0xfd,0x61,0xdb,0xf3,0x67	; (on hi)
	dt	0xb7,0xbf,0xe1,0xff,0xe7	;
	dt	0x02,0x00,0x01,0x02,0x02,0x02	;minus, blank, DP, (overflow)
;
LED_blanker
        addwf   PCL,F
        dt 	0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01
        dt	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
;------------------------------------------------------------
;
	cblock	0x20	
	temp_W				;
	temp_S				;
	at_rest_hi			;
	at_rest_lo			;
	gee_hi				;
	gee_lo				;
	buffer				;
	count_100th			;
	avg_count			;
;
	LED_matrix			;LED display regs
	rotor				;
	digit_count			;
	byte_0				;0x2C
	byte_1				;
	byte_2				;
	byte_3				;
	FLAGS				;
;
        ACCa_hi                 	;math registers
        ACCa_lo                 	;ACCa, b, c, d, e
        ACCb_hi				;
        ACCb_lo				;
        ACCc_hi				;
        ACCc_lo				;		
        ACCd_hi				;
        ACCd_lo				;
	math_count			;math counter
        hex_hi                  	;hex-decimal convert
	hex_lo				;
	ten_thousands			;
	thousands			;
	hundreds			;
        tens                    	;
        ones                   		; 
	endc
;
;------------------------------------------------------------
;
#define	AD_FLAG		FLAGS,0		;
;
#define	TP	PORTA,1			;test point
#define	I_0	PORTA,2			;LED 0 current sink (pin #9)
#define	I_1	PORTA,3			;LED 1 current sink (pin #10)
#define	I_2	PORTA,6			;LED 2 current sink (pin #11)
#define	I_3	PORTA,7			;LED 3 current sink (pin #12)
;
;
;	AD input set to analog
;	All other GPIO sink LED current when active
;
;	+1G = 2.974 volts @ PWM converted to analog out
;	-1G = 1.708
;	 0G = 2.341
;	 1g change = .632 volt ~ 129 counts A/D
;	 1 count = 0xc6(lsb) 
;
HUNDTH_SEC	equ	.2750		;1/100 sec timeout (tweaked 10%)
;
BLANK		equ	0x0b		;display nothing
MINUS_SIGN	equ	0x0a		;minus sign
;
ONE_CNT		equ	.198		;hex lsb value
;	
;------------------------------------------------------------
Bank0		MACRO			;macro to select data RAM bank 0
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank1		MACRO			;macro to select data RAM bank 1
		bsf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank2		MACRO			;macro to select data RAM bank 2
		bcf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM

Bank3		MACRO			;macro to select data RAM bank 3
		bsf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM

;------------------------------------------------------------
;
main
	Bank0
        movlw   0x07
        movwf   CMCON			;comparitors off
	clrf	PORTA
	clrf	PORTB
        Bank1			        ;switch to bank 1
        movlw   b'00000001'             ;port A in/outputs
        movwf   TRISA
        movlw   b'00000000'             ;port B in/outputs
        movwf   TRISB
        movlw	b'01111100'		;8 MHz int clock
        movwf	OSCCON
        movlw   b'00000001'             ;TMR0/4
        movwf   OPTION_REG
        movlw   b'00000001'             ;A:0 AD input
	movwf	ANSEL
        movlw   b'11000000'             ;right justify, /2
	movwf	ADCON1
        movlw   b'00000001'             ;AD off, TMR1 int on
        movwf   PIE1                    ;perif int setup
        Bank0		                ;back to page 0
;
        movlw   b'01000001'             ;
	movwf	ADCON0			;osc/16, on
        clrf    PIR1                    ;reset perif int
        movlw   b'00000000'             ;post/1, pre/1, off
        movwf   T2CON                   ;timer 2
        movlw   b'01100000'             ;PEIE on, TMR0IE on
	movwf	INTCON
	movlw	b'00110001'
	movwf	T1CON			;T1/8, on
;			
;------------------------------------------------------------
start_here
	clrf	rotor			;
	clrf	digit_count		;
	movlw	BLANK			;
	movwf	byte_0			;
	movwf	byte_1			;
	movwf	byte_2			;
	movwf	byte_3			;start all blanked
	clrf 	count_100th		;
	clrf	FLAGS			;
	bcf	TP			;test point low
	bsf	INTCON,GIE		;interrupts on
;
	call	LED_test		;test LEDs
;
	Bank1
	bsf	PIE1,ADIE
	Bank0
resting_read
	call	clear_math		;
	bsf	ADCON0,GO		;AD convert
	bcf	AD_FLAG
rr_loopb
	btfss	AD_FLAG
	goto	rr_loopb
	bcf	AD_FLAG
	movfw	gee_lo
	movwf	at_rest_lo
	movfw	gee_hi
	movwf	at_rest_hi		;buffer results
;
run_loop
	call	clear_math
	bsf	ADCON0,GO		;AD convert
run_loopb
	btfss	AD_FLAG
	goto	run_loopb
	bcf	AD_FLAG
;	
;
evaluation				
	call	clear_math	
	call	eval_acc		;rank the reading
;
show_results
	movfw	ones
	movwf	byte_0
	movfw	tens
	movwf	byte_1
	movfw	hundreds
	movwf	byte_2
	bcf	TP			;reset TP
	skpz
	bsf	TP			;hundreds = 1, TP hi	
	call	delay_01s
	call	delay_01s	
	goto	run_loop
;
;------------------------------------------------------------
;	TMR0 rolls over every 512 uS
;	TMR1 "		"     10,000 uS	 
;	
int_service
        movwf   temp_W
        swapf   STATUS,W
        movwf   temp_S
	Bank0	 			;bank 0
;
int_roster
	btfsc	INTCON,TMR0IF
	goto	T0_stuff
	btfsc	PIR1,ADIF
	goto	AD_stuff
	btfss	PIR1,TMR1IF
	goto	outta_here
;
T1_stuff
	bcf	PIR1,TMR1IF		;clear flag
	movlw	high HUNDTH_SEC
	subwf	TMR1H,F
	movlw	low HUNDTH_SEC
	subwf	TMR1L,F			;reload for .01 sec
	incf	count_100th,F		;count_100th + 1
	goto	int_roster
;
T0_stuff
	bcf	INTCON,TMR0IF		;reset	
	call	update_LED		;do display
	goto	int_roster
AD_stuff
	bcf	PIR1,ADIF		;clear AD int flag
	Bank1
	movfw	ADRESL
	Bank0
	movwf	gee_lo
	movfw	ADRESH
	andlw	b'00000011'
	movwf	gee_hi			;buffer results	
	bsf	AD_FLAG
	goto	int_roster
;
outta_here
        swapf   temp_S,W
        movwf   STATUS
        swapf   temp_W,F
        swapf   temp_W,W
	retfie
;
;------------------------------------------------------------
;	Subroutines	*************************************
;------------------------------------------------------------
;
;------------------------------------------------------------	
;
update_LED
	bsf	I_0			;all LED current off
	bsf	I_1
	bsf	I_2
	bsf	I_3
	movfw	rotor			;rotor to W
	call	LED_blanker		;look up bit to display
	andwf	LED_matrix,W		;select single bit (segment)
;
	movwf	PORTB			;hi/lo PORTB outputs
;
display_0
	movlw	.0
	xorwf	digit_count,W
	bnz	display_1
	bcf	I_0			;sink LED0 current
	goto	segment_shifter
display_1
	movlw	.1
	xorwf	digit_count,W
	bnz	display_2
	bcf	I_1			;sink LED1 current
	goto	segment_shifter
display_2
	movlw	.2
	xorwf	digit_count,W
	bnz	display_3
	bcf	I_2			;sink LED2 current
	goto	segment_shifter
display_3
	bcf	I_3			;sink LED3 current
;
segment_shifter
	incf	rotor,W			;rotor +1
	sublw	.7			; >7 ?
	bnc	reset_rotor		;yes
	incf	rotor,F			;no, inc
	return				; 
;
reset_rotor
	clrf	rotor			;rotor = 0
	incf	digit_count,F		;
	movlw	b'00000011'		;
	andwf	digit_count,F		;roll over at 3
digit_loader	
	movlw	byte_0			;byte_0 loc
	addwf	digit_count,W		;add offset (0-3)
	movwf	FSR			;lookup next byte
	movfw	INDF			;
	call	LED_digits		;get pattern
	movwf	LED_matrix		;new data loaded
	return
;
;------------------------------------------------------------
;
;	Compares current measurement to at rest measurement
;
eval_acc
	movfw	gee_hi
	subwf	at_rest_hi,W
	skpc				;gee_hi =< at_rest_hi
	goto	pos_gees		;gee_hi > at_rest_hi
	skpz
	goto	neg_gees		;gee_hi < at_rest_hi
;
	movfw	gee_lo			;gee_hi = at_rest_hi
	subwf	at_rest_lo,W		;check lo
	bnc	pos_gees		;gee_lo > at_rest_lo
;	
neg_gees
	movfw	at_rest_hi
	movwf	ACCb_hi
	movfw	at_rest_lo
	movwf	ACCb_lo
	movfw	gee_hi
	movwf	ACCa_hi
	movfw	gee_lo
	movwf	ACCa_lo
	call	subtract_16
	movlw	MINUS_SIGN
	movwf	byte_3			;leading minus
	goto	sort_results
;	
pos_gees
	movfw	gee_hi
	movwf	ACCb_hi
	movfw	gee_lo
	movwf	ACCb_lo
	movfw	at_rest_hi
	movwf	ACCa_hi
	movfw	at_rest_lo
	movwf	ACCa_lo
	call	subtract_16	
	movlw	BLANK
	movwf	byte_3			;clear any minus
;
;	Quantify the results for display
;
sort_results
	movlw	ONE_CNT			;mult factor x AD count
	movwf	ACCa_lo			
	clrf	ACCa_hi
	call	multiply_16
;
	movfw	ACCd_lo			;msb's contain result
	movwf	hex_hi
	movfw	ACCc_hi
	movwf	hex_lo
	call	hd_convert		;results now in decimal
	return	
;
;------------------------------------------------------------
;               Math Routines
;------------------------------------------------------------
clear_math
        clrf    ACCa_hi
        clrf    ACCa_lo
        clrf    ACCb_hi
        clrf    ACCb_lo
        clrf    ACCc_hi
        clrf    ACCc_lo
        clrf    ACCd_hi
        clrf    ACCd_lo
        return
;------------------------------------------------------------
;        16 x 16 Add and Subtract
;        ACCb_hi,lo - ACCa_hi,lo  result ACCb_hi,lo
subtract_16
        comf    ACCa_lo,F
        incf    ACCa_lo,F
        skpnz
        decf    ACCa_hi,F
        comf    ACCa_hi,F
;
;       ACCa_hi,lo + ACCb_hi,lo  result ACCb_hi,lo
add_16
        movfw   ACCa_lo
        addwf   ACCb_lo,F
        skpnc
        incf    ACCb_hi,F
        movfw   ACCa_hi
        addwf   ACCb_hi,F
        return
;------------------------------------------------------------
;	16x16 multiply
;	ACCa_hi,lo * ACCb_hi,lo
;	Results in ACCd_hi,lo ACCc_hi,lo
multiply_16
	clrf	ACCd_hi
	clrf	ACCd_lo
	clrf	ACCc_hi
	clrf	ACCc_lo
	bsf	ACCc_hi,7
multi_loop16
	rrf	ACCa_hi,F
	rrf	ACCa_lo,F
	skpc
	goto	no_add
	movfw	ACCb_lo
	addwf 	ACCd_lo,F
	movfw	ACCb_hi
	skpnc
	incfsz	ACCb_hi,W
	addwf	ACCd_hi,F
no_add
	rrf	ACCd_hi,F
	rrf	ACCd_lo,F
	rrf	ACCc_hi,F
	rrf	ACCc_lo,F
	skpc
	goto	multi_loop16
	return
;------------------------------------------------------------
;               16 x 16 divide
;               ACCb_hi,lo / ACCa_hi,lo
;               result in ACCb_hi,lo
;               remainder in ACCc_hi,lo
divide_16
        movlw   0x10
        movwf   math_count
        movfw   ACCb_hi
        movwf   ACCd_hi
        movfw   ACCb_lo
        movwf   ACCd_lo
        clrf    ACCb_hi
        clrf    ACCb_lo
        clrf    ACCc_hi
        clrf    ACCc_lo
div_loop
        clrc
        rlf     ACCd_lo,F
        rlf     ACCd_hi,F
        rlf     ACCc_lo,F
        rlf     ACCc_hi,F
        movfw   ACCa_hi
        subwf   ACCc_hi,W
        skpz
        goto    no_chk
        movfw   ACCa_lo
        subwf   ACCc_lo,W
no_chk
        skpc
        goto    no_go
        movfw   ACCa_lo
        subwf   ACCc_lo,F
        skpc
        decf    ACCc_hi,F
        movfw   ACCa_hi
        subwf   ACCc_hi,F
        setc
no_go
        rlf     ACCb_lo,F
        rlf     ACCb_hi,F
        decfsz  math_count,F
        goto    div_loop
        return
;
;------------------------------------------------------------
;	16 bit hex to decimal up to 65,535
;
hd_convert
        movlw   .0                    	;all zero (no ascii)
        movwf   ones
        movwf   tens
        movwf   hundreds
        movwf   thousands
	movwf	ten_thousands
;
tenthou_hd
	movlw	0x27
	subwf	hex_hi,W
	bnc	thou_hd
	movlw	0x27
	subwf	hex_hi,F
	movlw	0x10
	subwf	hex_lo,W
	bnc	tenthou_check
do_lo2
	movlw	0x10
	subwf	hex_lo,F
	incf	ten_thousands,F		;10,000's +1
	goto	tenthou_hd
tenthou_check
	movfw	hex_hi
	skpnz
	goto	add_back_10k
        decf    hex_hi,F
        goto    do_lo2
add_back_10k
	movlw	0x27
	addwf	hex_hi,F
;
thou_hd
        movlw   0x03
        subwf   hex_hi,W
        bnc     hund_hd
        movlw   0x03
        subwf   hex_hi,F
        movlw   0xe8
        subwf   hex_lo,W
        bnc     thou_check
do_lo1
        movlw   0xe8
        subwf   hex_lo,F
        incf    thousands,F             ;1000's + 1
        goto    thou_hd
thou_check
        movfw   hex_hi
        skpnz
        goto    add_back_1k
        decf    hex_hi,F
        goto    do_lo1
add_back_1k
        movlw   0x03
        addwf   hex_hi,F
;
hund_hd
        movlw   0x64
        subwf   hex_lo,W		;hex_hi < 100?
        bc      do_hund			;no
        movfw   hex_hi			;yes, can we carry?
        skpnz
        goto    tens_hd			;no
        decf    hex_hi,F		;yes
do_hund
        movlw   0x64
        subwf   hex_lo,F
        incf    hundreds,F             	;100's + 1
        goto    hund_hd
;
;	count how many times 10 can be subtracted from hex_lo
;
tens_hd
        movlw   0x0a
        subwf   hex_lo,W
        bnc     do_ones
        movlw   0x0a
        subwf   hex_lo,F	
        incf    tens,F                 	;10's + 1
        goto    tens_hd
do_ones
        movfw   hex_lo
        addwf   ones,F                 	;remainder = 1's
	clrf	hex_lo
        return
;
;------------------------------------------------------------
;	Counts down from 9 on all 4 digits
;
LED_test
	movlw	.9
	movwf	hex_lo			;start at 9
	movlw	.10
	movwf	hex_hi			;10 loops
LED_test_loop
	movfw	hex_lo
	movwf	byte_0			;load registers
	movwf	byte_1
	movwf	byte_2
	movwf	byte_3
	call	delay_1s		;1 second delay between
	decf	hex_lo,F
	decfsz	hex_hi,F
	goto	LED_test_loop	
	return			
;------------------------------------------------------------
;
;------------------------------------------------------------
;	1 second delay
;
delay_1s
	movfw	count_100th
	addlw	.100
	movwf	buffer
delay_1s_loop
	movfw	buffer	
	xorwf	count_100th,W
	skpz
	goto	delay_1s_loop
	return
;
;------------------------------------------------------------
;	.1 second delay
;
delay_01s
	movfw	count_100th
	addlw	.10
	movwf	buffer
delay_01s_loop
	movfw	buffer	
	xorwf	count_100th,W
	skpz
	goto	delay_01s_loop
	return
;	
;------------------------------------------------------------
;	Variable in W - 1 count = .01 second
;
variable_delay
	movwf	buffer
	movfw	count_100th
	addwf	buffer,F
variable_delay_loop
	movfw	buffer	
	xorwf	count_100th,W
	skpz
	goto	variable_delay_loop
	return	
;		
;------------------------------------------------------------
        end
;
;Copyright (c) 2008, Mike Berg aka N0QBH
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, 
;are permitted provided that the following conditions are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright notice, 
;this list of conditions and the following disclaimer in the documentation 
;and/or other materials provided with the distribution. 
;Neither the name of the <ringolake.com> nor the names of its contributors 
;may be used to endorse or promote products derived from this software without 
;specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
;AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
;THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
;PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
;BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
;SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
;INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
;ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;THE POSSIBILITY OF SUCH DAMAGE.		